package de.hft.stuttgart.citydoctor2.ziploader;

import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import org.apache.commons.io.FileUtils;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.mockito.Mockito;

import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.zip.ZipEntry;

import static org.junit.Assert.*;

public class ZipTest {

    ParserConfiguration config = new ParserConfiguration(8,false);

    @Rule
    public ExpectedException excRule = ExpectedException.none();

    @Test
    public void testUnzipping() {
        CityGmlArchive cgmlArch = CityGmlArchive.fromZipFile("src/test/resources/mock_archive.zip", config);
        assertNotNull(cgmlArch);
        assertEquals(5,cgmlArch.getEntries().size());
        for (CityGmlZipEntry entry : cgmlArch.getEntries()) {
            assertNotNull(entry);
            assertTrue(entry.getFileName().matches("^mock[1-5].gml$"));
            assertNotNull(entry.getModel());
        }
    }

    @Test
    public void testZippingWithFaultyFilepath(){
        CityGmlArchive cgmlArch = CityGmlArchive.fromZipFile("src/test/resources/mock_archive.zip", config);
        excRule.expect(IllegalArgumentException.class);
        excRule.expectMessage("Target zip-filepath must end with '.zip'");
        cgmlArch.exportToZipFile("src/test/resources/mock");
    }

    @Test
    public void testExcessiveFilesize(){
        ZipEntry zipEntry = Mockito.mock(ZipEntry.class);
        Path mockPath = Mockito.mock(Path.class);
        InputStream mockIs = Mockito.mock(InputStream.class);
        // Mock filesize of 30 Gb
        Mockito.when(zipEntry.getSize()).thenReturn(1024L*1024L*30);
        Mockito.when(zipEntry.getName()).thenReturn("mock.gml");
        CityGmlZipEntry cgmlZE = CityGmlZipEntry.of(zipEntry, config, "test", mockPath, mockIs);
        assertNotNull(cgmlZE);
        assertTrue(cgmlZE.isErroneousEntry());
        assertEquals(CityGmlZipEntry.ZipEntryErrorType.EXCESSIVE_FILESIZE, cgmlZE.getErrorType());

    }

    @Test
    public void testZipping() throws IOException {
        CityGmlArchive cgmlArch = CityGmlArchive.fromZipFile("src/test/resources/mock_archive.zip", config);
        Path tmpDir = null;
        try {
            tmpDir = Files.createTempDirectory("testTmp");
            tmpDir.toFile().deleteOnExit();
            String expPath = tmpDir.toString() + "/export.zip";
            cgmlArch.exportToZipFile(expPath);
            CityGmlArchive cgmlExport = CityGmlArchive.fromZipFile(expPath, config);
            assertNotNull(cgmlExport);
            assertEquals(5, cgmlExport.getEntries().size());
            for (CityGmlZipEntry entry : cgmlExport.getEntries()) {
                assertNotNull(entry);
                assertTrue(entry.getFileName().matches("^mock[1-5].gml$"));
                assertNotNull(entry.getModel());
            }
        } catch (Exception e) {
            // Rethrow Exceptions to ensure deletion of tmpDir with finally block
            throw e;
        } finally {
            if (tmpDir != null) {
                FileUtils.deleteDirectory(tmpDir.toFile());
            }
        }
    }

}
